import rclpy
from rclpy.node import Node
from transitions.extensions.asyncio import AsyncMachine
from std_msgs.msg import String
import asyncio
import threading
import time


class StatefulNode(Node):
    def __init__(self, event_loop):
        super().__init__('stateful_node')
        self.event_loop = event_loop
        self.subscription = self.create_subscription(String, 'my_topic', self.listener_callback, 10)

        states = [
            {'name': 'state1', 'on_enter': 'on_enter_state1'},
            {'name': 'state2', 'on_enter': 'on_enter_state2'},
            {'name': 'state3', 'on_enter': 'on_enter_state3'}
        ]
        transitions = [
            {'trigger': 'go_to_state1', 'source': '*', 'dest': 'state1'},
            {'trigger': 'go_to_state2', 'source': '*', 'dest': 'state2'},
            {'trigger': 'go_to_state3', 'source': '*', 'dest': 'state3'},
        ]

        self.machine = AsyncMachine(model=self, states=states, transitions=transitions, initial='state1')

    async def on_enter_state1(self):
        self.get_logger().info(f"Entering State 1 - {time.time()}")

    async def on_enter_state2(self):
        self.get_logger().info(f"Entering State 2 - {time.time()}")

    async def on_enter_state3(self):
        self.get_logger().info(f"Entering State 3 - {time.time()}")

    def listener_callback(self, msg):
        try:
            self.get_logger().info(f'Received message: "{msg.data}"')
            if msg.data == 'trigger1':
                self.get_logger().info("Attempting to go to state2")
                asyncio.run_coroutine_threadsafe(self.go_to_state1(), self.event_loop)
            elif msg.data == 'trigger2':
                self.get_logger().info("Attempting to go to state1")
                asyncio.run_coroutine_threadsafe(self.go_to_state2(), self.event_loop)
            elif msg.data == 'trigger3':
                self.get_logger().info("Attempting to go to state1")
                asyncio.run_coroutine_threadsafe(self.go_to_state3(), self.event_loop)
        except Exception as e:
            self.get_logger().error(f"Error in listener_callback: {str(e)}")


def run_asyncio_loop():
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    loop.run_forever()
    return loop  # Ensure you return the loop reference


def main():
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    threading.Thread(target=loop.run_forever, daemon=True).start()

    # Initialize ROS and pass the event loop to the node
    rclpy.init()
    node = StatefulNode(loop)

    # Now, the ROS 2 node has the correct event loop reference for asyncio operations
    rclpy.spin(node)

    node.destroy_node()
    rclpy.shutdown()
    loop.call_soon_threadsafe(loop.stop)

if __name__ == '__main__':
    main()