#!/usr/bin/env python3
import rclpy
from rclpy.node import Node

from .random_query import RandomQuery
from .regular_query import RegularQuery
from .improvement_query import ImprovementQuery

from interaction_msgs.srv import Query


class QueryNode(Node):
    def __init__(self):
        super().__init__('query_node')
        self.query_service = self.create_service(Query, 'user_query', self.query_callback)

        self.get_logger().info('Query node started!')

    def check_random_request(self, req):
        t = req.threshold
        if 0 < t <= 1:
            return True
        else:
            self.get_logger().error('Invalid random request in user query!')

    def check_regular_request(self, req):
        f = req.frequency
        if f > 0:
            return True
        else:
            self.get_logger().error('Invalid regular request in user query!')

    def check_improvement_request(self, req):
        t = req.threshold
        f = req.frequency
        last_rewards = req.last_rewards
        if 0 < t <= 1 and f > 0 and isinstance(last_rewards, list):
            return True
        else:
            self.get_logger().error('Invalid improvement request in user query!')

    def query_callback(self, request, response):
        mode = response.mode
        query_obj = None
        if mode == 0:
            if self.check_random_request(request):
                query_obj = RandomQuery(request.threshold)
        elif mode == 1:
            if self.check_regular_request(request):
                query_obj = RegularQuery(request.frequency, request.current_episode)
        elif mode == 2:
            if self.check_improvement_request(request):
                query_obj = ImprovementQuery(request.threshold, request.frequency,
                                             request.last_queried_episode, request.last_rewards)
        else:
            self.get_logger().error('Invalid query mode!')

        if query_obj is not None:
            response.interaction = query_obj.query()
            return response


def main(args=None):
    rclpy.init(args=args)
    node = QueryNode()
    rclpy.spin(node)
    rclpy.shutdown()


if __name__ == '__main__':
    main()
