import rclpy
from rclpy.node import Node
from rclpy.parameter import Parameter

from transitions import Machine
import yaml
import numpy as np
from movement_primitives.promp import ProMP
from src.interaction_utils.serialization import flatten_population, unflatten_population

from interaction_msgs.srv import TaskEvaluation
from std_msgs.msg import Bool


class TaskNode(Node):
    def __init__(self):
        super().__init__('task_node')

        # Task Attributes

        # ROS2 Interfaces

        # Heartbeat
        self.heartbeat_pub = self.create_publisher(Bool, 'interaction/task_heartbeat', 10)
        self.heartbeat_timer = self.create_timer(5.0, self.send_heartbeat)
        # Topic

        # Service

        # Action
        self._goal_handle = None

        # State Machine
        # States
        self.states = [
            'waiting_for_task_specs',
            'processing_non_interactive_input',
            'processing_interactive_inpute',
            'waiting_for_robot_response',
            'waiting_for_objective_function_response',
            'error_recovery'
        ]

        # initialize state machine
        self.machine = Machine(self, states=self.states, initial='waiting_for_task_specs')

        self.machine.add_transition(trigger='non_interactive_specs_received', source='waiting_for_task_specs', dest='processing_non_interactive_input')
        self.machine.add_transition(trigger='interactive_specs_received', source='waiting_for_task_specs', dest='processing_interactive_input')
        self.machine.add_transition(trigger='non_interactive_to_robot', source='processing_non_interactive_input', dest='waiting_for_robot_response')
        self.machine.add_transition(trigger='non_interactive_to_obj_fun', source='processing_non_interactive_input', dest='waiting_for_objective_function_response')
        self.machine.add_transition(trigger='interactive_to_robot', source='processing_interactive_input', dest='waiting_for_robot_response')
        self.machine.add_transition(trigger='interactive_to_obj_fun', source='processing_interactive_input', dest='waiting_for_objective_function_response')
        self.machine.add_transition(trigger='sending_robot_results', source='waiting_for_robot_response', dest='waiting_for_task_specs')
        self.machine.add_transition(trigger='sending_obj_fun_results', source='waiting_for_obj_fun_response', dest='waiting_for_task_specs')
        self.machine.add_transition(trigger='error_trigger', source='*', dest='error_recovery')
        self.machine.add_transition(trigger='recovery_complete', source='error_recovery', dest='waiting_for_task_specs')

    # State Methods

    # Callback functions
    def send_heartbeat(self):
        msg = Bool()
        self.heartbeat_pub.publish(msg)
